@@ -5,14 +5,17 @@ from esphome.const import (
     CONF_BATTERY_VOLTAGE,
     CONF_HUMIDITY,
     CONF_ID,
+    CONF_ILLUMINANCE,
     CONF_MOISTURE,
     CONF_MAC_ADDRESS,
     CONF_TEMPERATURE,
     DEVICE_CLASS_HUMIDITY,
+    DEVICE_CLASS_ILLUMINANCE,
     DEVICE_CLASS_TEMPERATURE,
     DEVICE_CLASS_VOLTAGE,
     STATE_CLASS_MEASUREMENT,
     UNIT_CELSIUS,
+    UNIT_LUX,
     UNIT_PERCENT,
     UNIT_VOLT,
 )
@@ -55,6 +58,12 @@ CONFIG_SCHEMA = (
                 device_class=DEVICE_CLASS_HUMIDITY,
                 state_class=STATE_CLASS_MEASUREMENT,
             ),
+            cv.Optional(CONF_ILLUMINANCE): sensor.sensor_schema(
+                unit_of_measurement=UNIT_LUX,
+                accuracy_decimals=0,
+                device_class=DEVICE_CLASS_ILLUMINANCE,
+                state_class=STATE_CLASS_MEASUREMENT,
+            ),
         }
     )
     .extend(esp32_ble_tracker.ESP_BLE_DEVICE_SCHEMA)
@@ -74,6 +83,7 @@ async def to_code(config):
         (CONF_HUMIDITY, var.set_humidity),
         (CONF_BATTERY_VOLTAGE, var.set_battery_voltage),
         (CONF_MOISTURE, var.set_soil_moisture),
+        (CONF_ILLUMINANCE, var.set_illuminance),
     ]:
         if config_key in config:
             sens = await sensor.new_sensor(config[config_key])
