@@ -1,16 +1,22 @@
 #include "logger.h"
 
-#ifdef ARDUINO_ARCH_ESP32
+#ifdef USE_ESP_IDF
+#include "freertos/FreeRTOS.h"
+#include <driver/uart.h>
+#endif
+
+#if defined(USE_ESP32_FRAMEWORK_ARDUINO) || defined(USE_ESP_IDF)
 #include <esp_log.h>
 #endif
-#include <HardwareSerial.h>
+#include "esphome/core/log.h"
+#include "esphome/core/hal.h"
 
 namespace esphome {
 namespace logger {
 
-static const char *TAG = "logger";
+static const char *const TAG = "logger";
 
-static const char *LOG_LEVEL_COLORS[] = {
+static const char *const LOG_LEVEL_COLORS[] = {
     "",                                            // NONE
     ESPHOME_LOG_BOLD(ESPHOME_LOG_COLOR_RED),       // ERROR
     ESPHOME_LOG_COLOR(ESPHOME_LOG_COLOR_YELLOW),   // WARNING
@@ -20,7 +26,7 @@ static const char *LOG_LEVEL_COLORS[] = {
     ESPHOME_LOG_COLOR(ESPHOME_LOG_COLOR_GRAY),     // VERBOSE
     ESPHOME_LOG_COLOR(ESPHOME_LOG_COLOR_WHITE),    // VERY_VERBOSE
 };
-static const char *LOG_LEVEL_LETTERS[] = {
+static const char *const LOG_LEVEL_LETTERS[] = {
     "",    // NONE
     "E",   // ERROR
     "W",   // WARNING
@@ -43,28 +49,31 @@ void Logger::write_header_(int level, const char *tag, int line) {
 }
 
 void HOT Logger::log_vprintf_(int level, const char *tag, int line, const char *format, va_list args) {  // NOLINT
-  if (level > this->level_for(tag))
+  if (level > this->level_for(tag) || recursion_guard_)
     return;
 
+  recursion_guard_ = true;
   this->reset_buffer_();
   this->write_header_(level, tag, line);
   this->vprintf_to_buffer_(format, args);
   this->write_footer_();
   this->log_message_(level, tag);
+  recursion_guard_ = false;
 }
 #ifdef USE_STORE_LOG_STR_IN_FLASH
 void Logger::log_vprintf_(int level, const char *tag, int line, const __FlashStringHelper *format,
                           va_list args) {  // NOLINT
-  if (level > this->level_for(tag))
+  if (level > this->level_for(tag) || recursion_guard_)
     return;
 
+  recursion_guard_ = true;
   this->reset_buffer_();
   // copy format string
-  const char *format_pgm_p = (PGM_P) format;
+  auto *format_pgm_p = reinterpret_cast<const uint8_t *>(format);
   size_t len = 0;
   char ch = '.';
   while (!this->is_buffer_full_() && ch != '\0') {
-    this->tx_buffer_[this->tx_buffer_at_++] = ch = pgm_read_byte(format_pgm_p++);
+    this->tx_buffer_[this->tx_buffer_at_++] = ch = (char) progmem_read_byte(format_pgm_p++);
   }
   // Buffer full form copying format
   if (this->is_buffer_full_())
@@ -78,6 +87,7 @@ void Logger::log_vprintf_(int level, const char *tag, int line, const __FlashStr
   this->vprintf_to_buffer_(this->tx_buffer_, args);
   this->write_footer_();
   this->log_message_(level, tag, offset);
+  recursion_guard_ = false;
 }
 #endif
 
@@ -101,32 +111,41 @@ void HOT Logger::log_message_(int level, const char *tag, int offset) {
   this->set_null_terminator_();
 
   const char *msg = this->tx_buffer_ + offset;
-  if (this->baud_rate_ > 0)
+  if (this->baud_rate_ > 0) {
+#ifdef USE_ARDUINO
     this->hw_serial_->println(msg);
-#ifdef ARDUINO_ARCH_ESP32
+#endif  // USE_ARDUINO
+#ifdef USE_ESP_IDF
+    uart_write_bytes(uart_num_, msg, strlen(msg));
+    uart_write_bytes(uart_num_, "\n", 1);
+#endif
+  }
+
+#ifdef USE_ESP32
   // Suppress network-logging if memory constrained, but still log to serial
   // ports. In some configurations (eg BLE enabled) there may be some transient
   // memory exhaustion, and trying to log when OOM can lead to a crash. Skipping
   // here usually allows the stack to recover instead.
   // See issue #1234 for analysis.
-  if (xPortGetFreeHeapSize() > 2048)
-    this->log_callback_.call(level, tag, msg);
-#else
-  this->log_callback_.call(level, tag, msg);
+  if (xPortGetFreeHeapSize() < 2048)
+    return;
 #endif
+
+  this->log_callback_.call(level, tag, msg);
 }
 
 Logger::Logger(uint32_t baud_rate, size_t tx_buffer_size, UARTSelection uart)
     : baud_rate_(baud_rate), tx_buffer_size_(tx_buffer_size), uart_(uart) {
   // add 1 to buffer size for null terminator
-  this->tx_buffer_ = new char[this->tx_buffer_size_ + 1];
+  this->tx_buffer_ = new char[this->tx_buffer_size_ + 1];  // NOLINT
 }
 
 void Logger::pre_setup() {
   if (this->baud_rate_ > 0) {
+#ifdef USE_ARDUINO
     switch (this->uart_) {
       case UART_SELECTION_UART0:
-#ifdef ARDUINO_ARCH_ESP8266
+#ifdef USE_ESP8266
       case UART_SELECTION_UART0_SWAP:
 #endif
         this->hw_serial_ = &Serial;
@@ -134,33 +153,58 @@ void Logger::pre_setup() {
       case UART_SELECTION_UART1:
         this->hw_serial_ = &Serial1;
         break;
-#ifdef ARDUINO_ARCH_ESP32
+#if defined(USE_ESP32) && !defined(USE_ESP32_VARIANT_ESP32C3) && !defined(USE_ESP32_VARIANT_ESP32S2)
       case UART_SELECTION_UART2:
-#if !CONFIG_IDF_TARGET_ESP32S2 && !CONFIG_IDF_TARGET_ESP32C3
-        // FIXME: Validate in config that UART2 can't be set for ESP32-S2 (only has
-        // UART0-UART1)
         this->hw_serial_ = &Serial2;
-#endif
         break;
 #endif
     }
+#endif  // USE_ARDUINO
+#ifdef USE_ESP_IDF
+    uart_num_ = UART_NUM_0;
+    switch (uart_) {
+      case UART_SELECTION_UART0:
+        uart_num_ = UART_NUM_0;
+        break;
+      case UART_SELECTION_UART1:
+        uart_num_ = UART_NUM_1;
+        break;
+#if defined(USE_ESP32) && !defined(USE_ESP32_VARIANT_ESP32C3) && !defined(USE_ESP32_VARIANT_ESP32S2)
+      case UART_SELECTION_UART2:
+        uart_num_ = UART_NUM_2;
+        break;
+#endif
+    }
+    uart_config_t uart_config{};
+    uart_config.baud_rate = (int) baud_rate_;
+    uart_config.data_bits = UART_DATA_8_BITS;
+    uart_config.parity = UART_PARITY_DISABLE;
+    uart_config.stop_bits = UART_STOP_BITS_1;
+    uart_config.flow_ctrl = UART_HW_FLOWCTRL_DISABLE;
+    uart_param_config(uart_num_, &uart_config);
+    const int uart_buffer_size = tx_buffer_size_;
+    // Install UART driver using an event queue here
+    uart_driver_install(uart_num_, uart_buffer_size, uart_buffer_size, 10, nullptr, 0);
+#endif
 
+#ifdef USE_ARDUINO
     this->hw_serial_->begin(this->baud_rate_);
-#ifdef ARDUINO_ARCH_ESP8266
+#ifdef USE_ESP8266
     if (this->uart_ == UART_SELECTION_UART0_SWAP) {
       this->hw_serial_->swap();
     }
     this->hw_serial_->setDebugOutput(ESPHOME_LOG_LEVEL >= ESPHOME_LOG_LEVEL_VERBOSE);
 #endif
+#endif  // USE_ARDUINO
   }
-#ifdef ARDUINO_ARCH_ESP8266
+#ifdef USE_ESP8266
   else {
     uart_set_debug(UART_NO);
   }
 #endif
 
   global_logger = this;
-#ifdef ARDUINO_ARCH_ESP32
+#if defined(USE_ESP_IDF) || defined(USE_ESP32_FRAMEWORK_ARDUINO)
   esp_log_set_vprintf(esp_idf_log_vprintf_);
   if (ESPHOME_LOG_LEVEL >= ESPHOME_LOG_LEVEL_VERBOSE) {
     esp_log_level_set("*", ESP_LOG_VERBOSE);
@@ -178,12 +222,12 @@ void Logger::add_on_log_callback(std::function<void(int, const char *, const cha
   this->log_callback_.add(std::move(callback));
 }
 float Logger::get_setup_priority() const { return setup_priority::HARDWARE - 1.0f; }
-const char *LOG_LEVELS[] = {"NONE", "ERROR", "WARN", "INFO", "CONFIG", "DEBUG", "VERBOSE", "VERY_VERBOSE"};
-#ifdef ARDUINO_ARCH_ESP32
-const char *UART_SELECTIONS[] = {"UART0", "UART1", "UART2"};
+const char *const LOG_LEVELS[] = {"NONE", "ERROR", "WARN", "INFO", "CONFIG", "DEBUG", "VERBOSE", "VERY_VERBOSE"};
+#ifdef USE_ESP32
+const char *const UART_SELECTIONS[] = {"UART0", "UART1", "UART2"};
 #endif
-#ifdef ARDUINO_ARCH_ESP8266
-const char *UART_SELECTIONS[] = {"UART0", "UART1", "UART0_SWAP"};
+#ifdef USE_ESP8266
+const char *const UART_SELECTIONS[] = {"UART0", "UART1", "UART0_SWAP"};
 #endif
 void Logger::dump_config() {
   ESP_LOGCONFIG(TAG, "Logger:");
@@ -196,7 +240,7 @@ void Logger::dump_config() {
 }
 void Logger::write_footer_() { this->write_to_buffer_(ESPHOME_LOG_RESET_COLOR, strlen(ESPHOME_LOG_RESET_COLOR)); }
 
-Logger *global_logger = nullptr;
+Logger *global_logger = nullptr;  // NOLINT(cppcoreguidelines-avoid-non-const-global-variables)
 
 }  // namespace logger
 }  // namespace esphome
