@@ -2,6 +2,7 @@ import esphome.codegen as cg
 import esphome.config_validation as cv
 from esphome.components import spi
 from esphome.const import CONF_ID
+from esphome.core import CORE
 
 DEPENDENCIES = ["spi"]
 AUTO_LOAD = ["sensor"]
@@ -19,7 +20,10 @@ CONFIG_SCHEMA = cv.Schema(
 ).extend(spi.spi_device_schema(cs_pin_required=True))
 
 
-def to_code(config):
+async def to_code(config):
     var = cg.new_Pvariable(config[CONF_ID])
-    yield cg.register_component(var, config)
-    yield spi.register_spi_device(var, config)
+    await cg.register_component(var, config)
+    await spi.register_spi_device(var, config)
+
+    if CORE.is_esp32:
+        cg.add_library("SPI", None)
