@@ -1,6 +1,7 @@
 #include "mqtt_light.h"
 #include "esphome/core/log.h"
 
+#ifdef USE_MQTT
 #ifdef USE_LIGHT
 
 #include "esphome/components/light/light_json_schema.h"
@@ -12,6 +13,7 @@ static const char *const TAG = "mqtt.light";
 using namespace esphome::light;
 
 std::string MQTTJSONLightComponent::component_type() const { return "light"; }
+const EntityBase *MQTTJSONLightComponent::get_entity() const { return this->state_; }
 
 void MQTTJSONLightComponent::setup() {
   this->subscribe_json(this->get_command_topic_(), [this](const std::string &topic, JsonObject &root) {
@@ -31,35 +33,34 @@ bool MQTTJSONLightComponent::publish_state_() {
                             [this](JsonObject &root) { LightJSONSchema::dump_json(*this->state_, root); });
 }
 LightState *MQTTJSONLightComponent::get_state() const { return this->state_; }
-std::string MQTTJSONLightComponent::friendly_name() const { return this->state_->get_name(); }
+
 void MQTTJSONLightComponent::send_discovery(JsonObject &root, mqtt::SendDiscoveryConfig &config) {
   root["schema"] = "json";
   auto traits = this->state_->get_traits();
 
   root["color_mode"] = true;
   JsonArray &color_modes = root.createNestedArray("supported_color_modes");
-  if (traits.supports_color_mode(ColorMode::COLOR_TEMPERATURE))
+  if (traits.supports_color_mode(ColorMode::ON_OFF))
+    color_modes.add("onoff");
+  if (traits.supports_color_mode(ColorMode::BRIGHTNESS))
+    color_modes.add("brightness");
+  if (traits.supports_color_mode(ColorMode::WHITE))
+    color_modes.add("white");
+  if (traits.supports_color_mode(ColorMode::COLOR_TEMPERATURE) ||
+      traits.supports_color_mode(ColorMode::COLD_WARM_WHITE))
     color_modes.add("color_temp");
   if (traits.supports_color_mode(ColorMode::RGB))
     color_modes.add("rgb");
-  if (traits.supports_color_mode(ColorMode::RGB_WHITE))
+  if (traits.supports_color_mode(ColorMode::RGB_WHITE) ||
+      // HA doesn't support RGBCT, and there's no CWWW->CT emulation in ESPHome yet, so ignore CT control for now
+      traits.supports_color_mode(ColorMode::RGB_COLOR_TEMPERATURE))
     color_modes.add("rgbw");
   if (traits.supports_color_mode(ColorMode::RGB_COLD_WARM_WHITE))
     color_modes.add("rgbww");
-  if (traits.supports_color_mode(ColorMode::BRIGHTNESS))
-    color_modes.add("brightness");
-  if (traits.supports_color_mode(ColorMode::ON_OFF))
-    color_modes.add("onoff");
 
   // legacy API
   if (traits.supports_color_capability(ColorCapability::BRIGHTNESS))
     root["brightness"] = true;
-  if (traits.supports_color_capability(ColorCapability::RGB))
-    root["rgb"] = true;
-  if (traits.supports_color_capability(ColorCapability::COLOR_TEMPERATURE))
-    root["color_temp"] = true;
-  if (traits.supports_color_capability(ColorCapability::WHITE))
-    root["white_value"] = true;
 
   if (this->state_->supports_effects()) {
     root["effect"] = true;
@@ -70,7 +71,6 @@ void MQTTJSONLightComponent::send_discovery(JsonObject &root, mqtt::SendDiscover
   }
 }
 bool MQTTJSONLightComponent::send_initial_state() { return this->publish_state_(); }
-bool MQTTJSONLightComponent::is_internal() { return this->state_->is_internal(); }
 void MQTTJSONLightComponent::dump_config() {
   ESP_LOGCONFIG(TAG, "MQTT Light '%s':", this->state_->get_name().c_str());
   LOG_MQTT_COMPONENT(true, true)
@@ -80,3 +80,4 @@ void MQTTJSONLightComponent::dump_config() {
 }  // namespace esphome
 
 #endif
+#endif  // USE_MQTT
