@@ -15,6 +15,7 @@ from esphome.core import CORE, coroutine_with_priority
 
 CODEOWNERS = ["@esphome/core"]
 DEPENDENCIES = ["network"]
+AUTO_LOAD = ["socket", "md5"]
 
 CONF_ON_STATE_CHANGE = "on_state_change"
 CONF_ON_BEGIN = "on_begin"
@@ -33,12 +34,13 @@ OTAProgressTrigger = ota_ns.class_("OTAProgressTrigger", automation.Trigger.temp
 OTAEndTrigger = ota_ns.class_("OTAEndTrigger", automation.Trigger.template())
 OTAErrorTrigger = ota_ns.class_("OTAErrorTrigger", automation.Trigger.template())
 
+
 CONFIG_SCHEMA = cv.Schema(
     {
         cv.GenerateID(): cv.declare_id(OTAComponent),
         cv.Optional(CONF_SAFE_MODE, default=True): cv.boolean,
         cv.SplitDefault(CONF_PORT, esp8266=8266, esp32=3232): cv.port,
-        cv.Optional(CONF_PASSWORD, default=""): cv.string,
+        cv.Optional(CONF_PASSWORD): cv.string,
         cv.Optional(
             CONF_REBOOT_TIMEOUT, default="5min"
         ): cv.positive_time_period_milliseconds,
@@ -76,7 +78,9 @@ CONFIG_SCHEMA = cv.Schema(
 async def to_code(config):
     var = cg.new_Pvariable(config[CONF_ID])
     cg.add(var.set_port(config[CONF_PORT]))
-    cg.add(var.set_auth_password(config[CONF_PASSWORD]))
+    if CONF_PASSWORD in config:
+        cg.add(var.set_auth_password(config[CONF_PASSWORD]))
+        cg.add_define("USE_OTA_PASSWORD")
 
     await cg.register_component(var, config)
 
@@ -88,8 +92,8 @@ async def to_code(config):
 
     if CORE.is_esp8266:
         cg.add_library("Update", None)
-    elif CORE.is_esp32:
-        cg.add_library("Hash", None)
+    elif CORE.is_esp32 and CORE.using_arduino:
+        cg.add_library("Update", None)
 
     use_state_callback = False
     for conf in config.get(CONF_ON_STATE_CHANGE, []):
