@@ -2,7 +2,7 @@
 #include "esphome/core/log.h"
 #include "esphome/core/helpers.h"
 
-#ifdef ARDUINO_ARCH_ESP32
+#ifdef USE_ESP32
 
 #include <vector>
 #include "mbedtls/ccm.h"
@@ -10,12 +10,12 @@
 namespace esphome {
 namespace xiaomi_ble {
 
-static const char *TAG = "xiaomi_ble";
+static const char *const TAG = "xiaomi_ble";
 
 bool parse_xiaomi_value(uint8_t value_type, const uint8_t *data, uint8_t value_length, XiaomiParseResult &result) {
   // motion detection, 1 byte, 8-bit unsigned integer
   if ((value_type == 0x03) && (value_length == 1)) {
-    result.has_motion = (data[0]) ? true : false;
+    result.has_motion = data[0];
   }
   // temperature, 2 bytes, 16-bit signed integer (LE), 0.1 °C
   else if ((value_type == 0x04) && (value_length == 2)) {
@@ -31,7 +31,7 @@ bool parse_xiaomi_value(uint8_t value_type, const uint8_t *data, uint8_t value_l
   else if (((value_type == 0x07) || (value_type == 0x0F)) && (value_length == 3)) {
     const uint32_t illuminance = uint32_t(data[0]) | (uint32_t(data[1]) << 8) | (uint32_t(data[2]) << 16);
     result.illuminance = illuminance;
-    result.is_light = (illuminance == 100) ? true : false;
+    result.is_light = illuminance == 100;
     if (value_type == 0x0F)
       result.has_motion = true;
   }
@@ -62,7 +62,7 @@ bool parse_xiaomi_value(uint8_t value_type, const uint8_t *data, uint8_t value_l
   }
   // on/off state, 1 byte, 8-bit unsigned integer
   else if ((value_type == 0x12) && (value_length == 1)) {
-    result.is_active = (data[0]) ? true : false;
+    result.is_active = data[0];
   }
   // mosquito tablet, 1 byte, 8-bit unsigned integer, 1 %
   else if ((value_type == 0x13) && (value_length == 1)) {
@@ -72,7 +72,7 @@ bool parse_xiaomi_value(uint8_t value_type, const uint8_t *data, uint8_t value_l
   else if ((value_type == 0x17) && (value_length == 4)) {
     const uint32_t idle_time = encode_uint32(data[3], data[2], data[1], data[0]);
     result.idle_time = idle_time / 60.0f;
-    result.has_motion = (idle_time) ? false : true;
+    result.has_motion = !idle_time;
   } else {
     return false;
   }
@@ -81,7 +81,7 @@ bool parse_xiaomi_value(uint8_t value_type, const uint8_t *data, uint8_t value_l
 }
 
 bool parse_xiaomi_message(const std::vector<uint8_t> &message, XiaomiParseResult &result) {
-  result.has_encryption = (message[0] & 0x08) ? true : false;  // update encryption status
+  result.has_encryption = message[0] & 0x08;  // update encryption status
   if (result.has_encryption) {
     ESP_LOGVV(TAG, "parse_xiaomi_message(): payload is encrypted, stop reading message.");
     return false;
@@ -104,7 +104,7 @@ bool parse_xiaomi_message(const std::vector<uint8_t> &message, XiaomiParseResult
   }
 
   while (payload_length > 3) {
-    if (payload[payload_offset + 1] != 0x10) {
+    if (payload[payload_offset + 1] != 0x10 && payload[payload_offset + 1] != 0x00) {
       ESP_LOGVV(TAG, "parse_xiaomi_message(): fixed byte not found, stop parsing residual data.");
       break;
     }
@@ -136,9 +136,9 @@ optional<XiaomiParseResult> parse_xiaomi_header(const esp32_ble_tracker::Service
   }
 
   auto raw = service_data.data;
-  result.has_data = (raw[0] & 0x40) ? true : false;
-  result.has_capability = (raw[0] & 0x20) ? true : false;
-  result.has_encryption = (raw[0] & 0x08) ? true : false;
+  result.has_data = raw[0] & 0x40;
+  result.has_capability = raw[0] & 0x20;
+  result.has_encryption = raw[0] & 0x08;
 
   if (!result.has_data) {
     ESP_LOGVV(TAG, "parse_xiaomi_header(): service data has no DATA flag.");
@@ -189,6 +189,9 @@ optional<XiaomiParseResult> parse_xiaomi_header(const esp32_ble_tracker::Service
   } else if ((raw[2] == 0x76) && (raw[3] == 0x05)) {  // Cleargrass (Qingping) alarm clock, segment LCD
     result.type = XiaomiParseResult::TYPE_CGD1;
     result.name = "CGD1";
+  } else if ((raw[2] == 0x6F) && (raw[3] == 0x06)) {  // Cleargrass (Qingping) Temp & RH Lite
+    result.type = XiaomiParseResult::TYPE_CGDK2;
+    result.name = "CGDK2";
   } else if ((raw[2] == 0x5b) && (raw[3] == 0x05)) {  // small square body, segment LCD, encrypted
     result.type = XiaomiParseResult::TYPE_LYWSD03MMC;
     result.name = "LYWSD03MMC";
@@ -200,6 +203,11 @@ optional<XiaomiParseResult> parse_xiaomi_header(const esp32_ble_tracker::Service
   } else if ((raw[2] == 0x87) && (raw[3] == 0x03)) {  // square body, e-ink display
     result.type = XiaomiParseResult::TYPE_MHOC401;
     result.name = "MHOC401";
+  } else if ((raw[2] == 0x83) && (raw[3] == 0x0A)) {  // Qingping-branded, motion & ambient light sensor
+    result.type = XiaomiParseResult::TYPE_CGPR1;
+    result.name = "CGPR1";
+    if (raw.size() == 19)
+      result.raw_offset -= 6;
   } else {
     ESP_LOGVV(TAG, "parse_xiaomi_header(): unknown device, no magic bytes.");
     return {};
@@ -341,9 +349,9 @@ bool report_xiaomi_results(const optional<XiaomiParseResult> &result, const std:
 bool XiaomiListener::parse_device(const esp32_ble_tracker::ESPBTDevice &device) {
   // Previously the message was parsed twice per packet, once by XiaomiListener::parse_device()
   // and then again by the respective device class's parse_device() function. Parsing the header
-  // here and then for each device seems to be unneccessary and complicates the duplicate packet filtering.
+  // here and then for each device seems to be unnecessary and complicates the duplicate packet filtering.
   // Hence I disabled the call to parse_xiaomi_header() here and the message parsing is done entirely
-  // in the respecive device instance. The XiaomiListener class is defined in __init__.py and I was not
+  // in the respective device instance. The XiaomiListener class is defined in __init__.py and I was not
   // able to remove it entirely.
 
   return false;  // with true it's not showing device scans
