@@ -1,26 +1,12 @@
 """Constants used by esphome."""
 
-__version__ = "2021.8.0"
-
-ESP_PLATFORM_ESP32 = "ESP32"
-ESP_PLATFORM_ESP8266 = "ESP8266"
-ESP_PLATFORMS = [ESP_PLATFORM_ESP32, ESP_PLATFORM_ESP8266]
+__version__ = "2021.10.0"
 
 ALLOWED_NAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789-_"
 
-# Lookup table from ESP32 arduino framework version to latest platformio
-# package with that version
-# See also https://github.com/platformio/platform-espressif32/releases
-ARDUINO_VERSION_ESP32 = {
-    "dev": "https://github.com/platformio/platform-espressif32.git",
-    "1.0.6": "platformio/espressif32@3.2.0",
-    "1.0.5": "platformio/espressif32@3.1.1",
-    "1.0.4": "platformio/espressif32@3.0.0",
-    "1.0.3": "platformio/espressif32@1.10.0",
-    "1.0.2": "platformio/espressif32@1.9.0",
-    "1.0.1": "platformio/espressif32@1.7.0",
-    "1.0.0": "platformio/espressif32@1.5.0",
-}
+TARGET_PLATFORMS = ["esp32", "esp8266"]
+TARGET_FRAMEWORKS = ["arduino", "esp-idf"]
+
 # See also https://github.com/platformio/platform-espressif8266/releases
 ARDUINO_VERSION_ESP8266 = {
     "dev": "https://github.com/platformio/platform-espressif8266.git",
@@ -53,9 +39,11 @@ CONF_ACCELERATION_Z = "acceleration_z"
 CONF_ACCURACY = "accuracy"
 CONF_ACCURACY_DECIMALS = "accuracy_decimals"
 CONF_ACTION_ID = "action_id"
+CONF_ACTION_STATE_TOPIC = "action_state_topic"
 CONF_ACTIVE_POWER = "active_power"
 CONF_ADDRESS = "address"
 CONF_ADDRESSABLE_LIGHT_ID = "addressable_light_id"
+CONF_ADVANCED = "advanced"
 CONF_ALPHA = "alpha"
 CONF_ALTITUDE = "altitude"
 CONF_AND = "and"
@@ -69,15 +57,19 @@ CONF_ATTENUATION = "attenuation"
 CONF_ATTRIBUTE = "attribute"
 CONF_AUTH = "auth"
 CONF_AUTO_MODE = "auto_mode"
+CONF_AUTOCONF = "autoconf"
 CONF_AUTOMATION_ID = "automation_id"
 CONF_AVAILABILITY = "availability"
 CONF_AWAY = "away"
+CONF_AWAY_COMMAND_TOPIC = "away_command_topic"
 CONF_AWAY_CONFIG = "away_config"
+CONF_AWAY_STATE_TOPIC = "away_state_topic"
 CONF_BACKLIGHT_PIN = "backlight_pin"
 CONF_BASELINE = "baseline"
 CONF_BATTERY_LEVEL = "battery_level"
 CONF_BATTERY_VOLTAGE = "battery_voltage"
 CONF_BAUD_RATE = "baud_rate"
+CONF_BEEPER = "beeper"
 CONF_BELOW = "below"
 CONF_BINARY = "binary"
 CONF_BINARY_SENSOR = "binary_sensor"
@@ -88,6 +80,7 @@ CONF_BIT_DEPTH = "bit_depth"
 CONF_BLUE = "blue"
 CONF_BOARD = "board"
 CONF_BOARD_FLASH_MODE = "board_flash_mode"
+CONF_BORDER = "border"
 CONF_BRANCH = "branch"
 CONF_BRIGHTNESS = "brightness"
 CONF_BROKER = "broker"
@@ -122,6 +115,7 @@ CONF_COLD_WHITE_COLOR_TEMPERATURE = "cold_white_color_temperature"
 CONF_COLOR = "color"
 CONF_COLOR_BRIGHTNESS = "color_brightness"
 CONF_COLOR_CORRECT = "color_correct"
+CONF_COLOR_INTERLOCK = "color_interlock"
 CONF_COLOR_MODE = "color_mode"
 CONF_COLOR_TEMPERATURE = "color_temperature"
 CONF_COLORS = "colors"
@@ -150,6 +144,7 @@ CONF_CSS_URL = "css_url"
 CONF_CURRENT = "current"
 CONF_CURRENT_OPERATION = "current_operation"
 CONF_CURRENT_RESISTOR = "current_resistor"
+CONF_CURRENT_TEMPERATURE_STATE_TOPIC = "current_temperature_state_topic"
 CONF_CUSTOM_FAN_MODE = "custom_fan_mode"
 CONF_CUSTOM_FAN_MODES = "custom_fan_modes"
 CONF_CUSTOM_PRESET = "custom_preset"
@@ -165,6 +160,7 @@ CONF_DAYS_OF_WEEK = "days_of_week"
 CONF_DC_PIN = "dc_pin"
 CONF_DEASSERT_RTS_DTR = "deassert_rts_dtr"
 CONF_DEBOUNCE = "debounce"
+CONF_DECAY_MODE = "decay_mode"
 CONF_DECELERATION = "deceleration"
 CONF_DEFAULT_MODE = "default_mode"
 CONF_DEFAULT_TARGET_TEMPERATURE_HIGH = "default_target_temperature_high"
@@ -200,12 +196,11 @@ CONF_ECO2 = "eco2"
 CONF_EFFECT = "effect"
 CONF_EFFECTS = "effects"
 CONF_ELSE = "else"
-CONF_ENABLE_MDNS = "enable_mdns"
 CONF_ENABLE_PIN = "enable_pin"
 CONF_ENABLE_TIME = "enable_time"
 CONF_ENERGY = "energy"
 CONF_ENTITY_ID = "entity_id"
-CONF_ESP8266_RESTORE_FROM_FLASH = "esp8266_restore_from_flash"
+CONF_ESP8266_DISABLE_SSL_SUPPORT = "esp8266_disable_ssl_support"
 CONF_ESPHOME = "esphome"
 CONF_ETHERNET = "ethernet"
 CONF_EVENT = "event"
@@ -218,6 +213,7 @@ CONF_FALLING_EDGE = "falling_edge"
 CONF_FAMILY = "family"
 CONF_FAN_MODE = "fan_mode"
 CONF_FAN_MODE_AUTO_ACTION = "fan_mode_auto_action"
+CONF_FAN_MODE_COMMAND_TOPIC = "fan_mode_command_topic"
 CONF_FAN_MODE_DIFFUSE_ACTION = "fan_mode_diffuse_action"
 CONF_FAN_MODE_FOCUS_ACTION = "fan_mode_focus_action"
 CONF_FAN_MODE_HIGH_ACTION = "fan_mode_high_action"
@@ -226,6 +222,7 @@ CONF_FAN_MODE_MEDIUM_ACTION = "fan_mode_medium_action"
 CONF_FAN_MODE_MIDDLE_ACTION = "fan_mode_middle_action"
 CONF_FAN_MODE_OFF_ACTION = "fan_mode_off_action"
 CONF_FAN_MODE_ON_ACTION = "fan_mode_on_action"
+CONF_FAN_MODE_STATE_TOPIC = "fan_mode_state_topic"
 CONF_FAN_ONLY_ACTION = "fan_only_action"
 CONF_FAN_ONLY_ACTION_USES_FAN_MODE_TIMER = "fan_only_action_uses_fan_mode_timer"
 CONF_FAN_ONLY_COOLING = "fan_only_cooling"
@@ -234,18 +231,21 @@ CONF_FAN_WITH_COOLING = "fan_with_cooling"
 CONF_FAN_WITH_HEATING = "fan_with_heating"
 CONF_FAST_CONNECT = "fast_connect"
 CONF_FILE = "file"
+CONF_FILES = "files"
 CONF_FILTER = "filter"
 CONF_FILTER_OUT = "filter_out"
 CONF_FILTERS = "filters"
 CONF_FINGER_ID = "finger_id"
 CONF_FINGERPRINT_COUNT = "fingerprint_count"
 CONF_FLASH_LENGTH = "flash_length"
+CONF_FLASH_TRANSITION_LENGTH = "flash_transition_length"
 CONF_FLOW_CONTROL_PIN = "flow_control_pin"
 CONF_FOR = "for"
 CONF_FORCE_UPDATE = "force_update"
 CONF_FORMALDEHYDE = "formaldehyde"
 CONF_FORMAT = "format"
 CONF_FORWARD_ACTIVE_ENERGY = "forward_active_energy"
+CONF_FRAMEWORK = "framework"
 CONF_FREQUENCY = "frequency"
 CONF_FROM = "from"
 CONF_FULL_SPECTRUM = "full_spectrum"
@@ -277,6 +277,9 @@ CONF_HUMIDITY = "humidity"
 CONF_HYSTERESIS = "hysteresis"
 CONF_I2C = "i2c"
 CONF_I2C_ID = "i2c_id"
+CONF_IBEACON_MAJOR = "ibeacon_major"
+CONF_IBEACON_MINOR = "ibeacon_minor"
+CONF_IBEACON_UUID = "ibeacon_uuid"
 CONF_ICON = "icon"
 CONF_ID = "id"
 CONF_IDENTITY = "identity"
@@ -285,6 +288,7 @@ CONF_IDLE_ACTION = "idle_action"
 CONF_IDLE_LEVEL = "idle_level"
 CONF_IDLE_TIME = "idle_time"
 CONF_IF = "if"
+CONF_IGNORE_EFUSE_MAC_CRC = "ignore_efuse_mac_crc"
 CONF_IIR_FILTER = "iir_filter"
 CONF_ILLUMINANCE = "illuminance"
 CONF_IMPEDANCE = "impedance"
@@ -297,6 +301,7 @@ CONF_INFRARED = "infrared"
 CONF_INITIAL_MODE = "initial_mode"
 CONF_INITIAL_OPTION = "initial_option"
 CONF_INITIAL_VALUE = "initial_value"
+CONF_INPUT = "input"
 CONF_INTEGRATION_TIME = "integration_time"
 CONF_INTENSITY = "intensity"
 CONF_INTERLOCK = "interlock"
@@ -317,15 +322,18 @@ CONF_KEY = "key"
 CONF_LAMBDA = "lambda"
 CONF_LAST_CONFIDENCE = "last_confidence"
 CONF_LAST_FINGER_ID = "last_finger_id"
-CONF_LAST_RESET_TYPE = "last_reset_type"
 CONF_LATITUDE = "latitude"
+CONF_LEGEND = "legend"
 CONF_LENGTH = "length"
 CONF_LEVEL = "level"
 CONF_LG = "lg"
 CONF_LIBRARIES = "libraries"
 CONF_LIGHT = "light"
+CONF_LIGHT_ID = "light_id"
 CONF_LIGHTNING_ENERGY = "lightning_energy"
 CONF_LIGHTNING_THRESHOLD = "lightning_threshold"
+CONF_LINE_THICKNESS = "line_thickness"
+CONF_LINE_TYPE = "line_type"
 CONF_LOADED_INTEGRATIONS = "loaded_integrations"
 CONF_LOCAL = "local"
 CONF_LOG_TOPIC = "log_topic"
@@ -347,6 +355,7 @@ CONF_MAX_HEATING_RUN_TIME = "max_heating_run_time"
 CONF_MAX_LENGTH = "max_length"
 CONF_MAX_LEVEL = "max_level"
 CONF_MAX_POWER = "max_power"
+CONF_MAX_RANGE = "max_range"
 CONF_MAX_REFRESH_RATE = "max_refresh_rate"
 CONF_MAX_SPEED = "max_speed"
 CONF_MAX_TEMPERATURE = "max_temperature"
@@ -368,12 +377,15 @@ CONF_MIN_IDLE_TIME = "min_idle_time"
 CONF_MIN_LENGTH = "min_length"
 CONF_MIN_LEVEL = "min_level"
 CONF_MIN_POWER = "min_power"
+CONF_MIN_RANGE = "min_range"
 CONF_MIN_TEMPERATURE = "min_temperature"
 CONF_MIN_VALUE = "min_value"
 CONF_MINUTE = "minute"
 CONF_MINUTES = "minutes"
 CONF_MISO_PIN = "miso_pin"
 CONF_MODE = "mode"
+CONF_MODE_COMMAND_TOPIC = "mode_command_topic"
+CONF_MODE_STATE_TOPIC = "mode_state_topic"
 CONF_MODEL = "model"
 CONF_MOISTURE = "moisture"
 CONF_MONTHS = "months"
@@ -385,6 +397,7 @@ CONF_MQTT_ID = "mqtt_id"
 CONF_MULTIPLEXER = "multiplexer"
 CONF_MULTIPLY = "multiply"
 CONF_NAME = "name"
+CONF_NAME_FONT = "name_font"
 CONF_NBITS = "nbits"
 CONF_NEC = "nec"
 CONF_NETWORKS = "networks"
@@ -420,6 +433,7 @@ CONF_ON_PRESS = "on_press"
 CONF_ON_RAW_VALUE = "on_raw_value"
 CONF_ON_RELEASE = "on_release"
 CONF_ON_SHUTDOWN = "on_shutdown"
+CONF_ON_SPEED_SET = "on_speed_set"
 CONF_ON_STATE = "on_state"
 CONF_ON_TAG = "on_tag"
 CONF_ON_TAG_REMOVED = "on_tag_removed"
@@ -431,6 +445,7 @@ CONF_ON_VALUE = "on_value"
 CONF_ON_VALUE_RANGE = "on_value_range"
 CONF_ONE = "one"
 CONF_OPEN_ACTION = "open_action"
+CONF_OPEN_DRAIN = "open_drain"
 CONF_OPEN_DRAIN_INTERRUPT = "open_drain_interrupt"
 CONF_OPEN_DURATION = "open_duration"
 CONF_OPEN_ENDSTOP = "open_endstop"
@@ -490,6 +505,8 @@ CONF_PMC_4_0 = "pmc_4_0"
 CONF_PORT = "port"
 CONF_POSITION = "position"
 CONF_POSITION_ACTION = "position_action"
+CONF_POSITION_COMMAND_TOPIC = "position_command_topic"
+CONF_POSITION_STATE_TOPIC = "position_state_topic"
 CONF_POWER = "power"
 CONF_POWER_FACTOR = "power_factor"
 CONF_POWER_ON_VALUE = "power_on_value"
@@ -504,22 +521,29 @@ CONF_PRIORITY = "priority"
 CONF_PROJECT = "project"
 CONF_PROTOCOL = "protocol"
 CONF_PULL_MODE = "pull_mode"
+CONF_PULLDOWN = "pulldown"
+CONF_PULLUP = "pullup"
 CONF_PULSE_LENGTH = "pulse_length"
 CONF_QOS = "qos"
+CONF_RADON = "radon"
+CONF_RADON_LONG_TERM = "radon_long_term"
 CONF_RANDOM = "random"
 CONF_RANGE = "range"
 CONF_RANGE_FROM = "range_from"
 CONF_RANGE_TO = "range_to"
 CONF_RATE = "rate"
 CONF_RAW = "raw"
+CONF_RAW_DATA_ID = "raw_data_id"
 CONF_RC_CODE_1 = "rc_code_1"
 CONF_RC_CODE_2 = "rc_code_2"
 CONF_REACTIVE_POWER = "reactive_power"
 CONF_REBOOT_TIMEOUT = "reboot_timeout"
 CONF_RECEIVE_TIMEOUT = "receive_timeout"
 CONF_RED = "red"
+CONF_REF = "ref"
 CONF_REFERENCE_RESISTANCE = "reference_resistance"
 CONF_REFERENCE_TEMPERATURE = "reference_temperature"
+CONF_REFRESH = "refresh"
 CONF_REPEAT = "repeat"
 CONF_REPOSITORY = "repository"
 CONF_RESET_PIN = "reset_pin"
@@ -548,6 +572,7 @@ CONF_SAFE_MODE = "safe_mode"
 CONF_SAMSUNG = "samsung"
 CONF_SATELLITES = "satellites"
 CONF_SCAN = "scan"
+CONF_SCAN_RESULTS = "scan_results"
 CONF_SCL = "scl"
 CONF_SCL_PIN = "scl_pin"
 CONF_SDA = "sda"
@@ -571,9 +596,13 @@ CONF_SERVICES = "services"
 CONF_SET_POINT_MINIMUM_DIFFERENTIAL = "set_point_minimum_differential"
 CONF_SETUP_MODE = "setup_mode"
 CONF_SETUP_PRIORITY = "setup_priority"
+CONF_SHOW_LINES = "show_lines"
+CONF_SHOW_UNITS = "show_units"
+CONF_SHOW_VALUES = "show_values"
 CONF_SHUNT_RESISTANCE = "shunt_resistance"
 CONF_SHUNT_VOLTAGE = "shunt_voltage"
 CONF_SHUTDOWN_MESSAGE = "shutdown_message"
+CONF_SINGLE_LIGHT_ID = "single_light_id"
 CONF_SIZE = "size"
 CONF_SLEEP_DURATION = "sleep_duration"
 CONF_SLEEP_PIN = "sleep_pin"
@@ -583,6 +612,8 @@ CONF_SOURCE = "source"
 CONF_SPEED = "speed"
 CONF_SPEED_COMMAND_TOPIC = "speed_command_topic"
 CONF_SPEED_COUNT = "speed_count"
+CONF_SPEED_LEVEL_COMMAND_TOPIC = "speed_level_command_topic"
+CONF_SPEED_LEVEL_STATE_TOPIC = "speed_level_state_topic"
 CONF_SPEED_STATE_TOPIC = "speed_state_topic"
 CONF_SPI_ID = "spi_id"
 CONF_SPIKE_REJECTION = "spike_rejection"
@@ -605,11 +636,17 @@ CONF_SUPPLEMENTAL_COOLING_ACTION = "supplemental_cooling_action"
 CONF_SUPPLEMENTAL_COOLING_DELTA = "supplemental_cooling_delta"
 CONF_SUPPLEMENTAL_HEATING_ACTION = "supplemental_heating_action"
 CONF_SUPPLEMENTAL_HEATING_DELTA = "supplemental_heating_delta"
+CONF_SUPPORTED_FAN_MODES = "supported_fan_modes"
+CONF_SUPPORTED_MODES = "supported_modes"
+CONF_SUPPORTED_PRESETS = "supported_presets"
+CONF_SUPPORTED_SWING_MODES = "supported_swing_modes"
 CONF_SUPPORTS_COOL = "supports_cool"
 CONF_SUPPORTS_HEAT = "supports_heat"
 CONF_SWING_BOTH_ACTION = "swing_both_action"
 CONF_SWING_HORIZONTAL_ACTION = "swing_horizontal_action"
 CONF_SWING_MODE = "swing_mode"
+CONF_SWING_MODE_COMMAND_TOPIC = "swing_mode_command_topic"
+CONF_SWING_MODE_STATE_TOPIC = "swing_mode_state_topic"
 CONF_SWING_OFF_ACTION = "swing_off_action"
 CONF_SWING_VERTICAL_ACTION = "swing_vertical_action"
 CONF_SWITCH_DATAPOINT = "switch_datapoint"
@@ -620,8 +657,14 @@ CONF_TAG = "tag"
 CONF_TARGET = "target"
 CONF_TARGET_TEMPERATURE = "target_temperature"
 CONF_TARGET_TEMPERATURE_CHANGE_ACTION = "target_temperature_change_action"
+CONF_TARGET_TEMPERATURE_COMMAND_TOPIC = "target_temperature_command_topic"
 CONF_TARGET_TEMPERATURE_HIGH = "target_temperature_high"
+CONF_TARGET_TEMPERATURE_HIGH_COMMAND_TOPIC = "target_temperature_high_command_topic"
+CONF_TARGET_TEMPERATURE_HIGH_STATE_TOPIC = "target_temperature_high_state_topic"
 CONF_TARGET_TEMPERATURE_LOW = "target_temperature_low"
+CONF_TARGET_TEMPERATURE_LOW_COMMAND_TOPIC = "target_temperature_low_command_topic"
+CONF_TARGET_TEMPERATURE_LOW_STATE_TOPIC = "target_temperature_low_state_topic"
+CONF_TARGET_TEMPERATURE_STATE_TOPIC = "target_temperature_state_topic"
 CONF_TEMPERATURE = "temperature"
 CONF_TEMPERATURE_STEP = "temperature_step"
 CONF_TEXT_SENSORS = "text_sensors"
@@ -630,7 +673,9 @@ CONF_THRESHOLD = "threshold"
 CONF_THROTTLE = "throttle"
 CONF_TILT = "tilt"
 CONF_TILT_ACTION = "tilt_action"
+CONF_TILT_COMMAND_TOPIC = "tilt_command_topic"
 CONF_TILT_LAMBDA = "tilt_lambda"
+CONF_TILT_STATE_TOPIC = "tilt_state_topic"
 CONF_TIME = "time"
 CONF_TIME_ID = "time_id"
 CONF_TIMEOUT = "timeout"
@@ -642,6 +687,7 @@ CONF_TOLERANCE = "tolerance"
 CONF_TOPIC = "topic"
 CONF_TOPIC_PREFIX = "topic_prefix"
 CONF_TOTAL = "total"
+CONF_TRACES = "traces"
 CONF_TRANSITION_LENGTH = "transition_length"
 CONF_TRIGGER_ID = "trigger_id"
 CONF_TRIGGER_PIN = "trigger_pin"
@@ -664,6 +710,7 @@ CONF_USE_ADDRESS = "use_address"
 CONF_USERNAME = "username"
 CONF_UUID = "uuid"
 CONF_VALUE = "value"
+CONF_VALUE_FONT = "value_font"
 CONF_VARIABLES = "variables"
 CONF_VARIANT = "variant"
 CONF_VERSION = "version"
@@ -687,6 +734,8 @@ CONF_WILL_MESSAGE = "will_message"
 CONF_WIND_DIRECTION_DEGREES = "wind_direction_degrees"
 CONF_WIND_SPEED = "wind_speed"
 CONF_WINDOW_SIZE = "window_size"
+CONF_X_GRID = "x_grid"
+CONF_Y_GRID = "y_grid"
 CONF_ZERO = "zero"
 
 ENV_NOGITIGNORE = "ESPHOME_NOGITIGNORE"
@@ -731,7 +780,9 @@ ICON_PERCENT = "mdi:percent"
 ICON_POWER = "mdi:power"
 ICON_PULSE = "mdi:pulse"
 ICON_RADIATOR = "mdi:radiator"
+ICON_RADIOACTIVE = "mdi:radioactive"
 ICON_RESTART = "mdi:restart"
+ICON_RESTART_ALERT = "mdi:restart-alert"
 ICON_ROTATE_RIGHT = "mdi:rotate-right"
 ICON_RULER = "mdi:ruler"
 ICON_SCALE = "mdi:scale"
@@ -752,6 +803,7 @@ ICON_WEATHER_WINDY = "mdi:weather-windy"
 ICON_WIFI = "mdi:wifi"
 
 UNIT_AMPERE = "A"
+UNIT_BECQUEREL_PER_CUBIC_METER = "Bq/m³"
 UNIT_CELSIUS = "°C"
 UNIT_COUNT_DECILITRE = "/dL"
 UNIT_COUNTS_PER_CUBIC_METER = "#/m³"
@@ -768,6 +820,10 @@ UNIT_KELVIN = "K"
 UNIT_KILOGRAM = "kg"
 UNIT_KILOMETER = "km"
 UNIT_KILOMETER_PER_HOUR = "km/h"
+UNIT_KILOVOLT_AMPS_REACTIVE = "kVAr"
+UNIT_KILOVOLT_AMPS_REACTIVE_HOURS = "kVArh"
+UNIT_KILOWATT = "kW"
+UNIT_KILOWATT_HOURS = "kWh"
 UNIT_LUX = "lx"
 UNIT_METER = "m"
 UNIT_METER_PER_SECOND_SQUARED = "m/s²"
@@ -798,7 +854,6 @@ DEVICE_CLASS_COLD = "cold"
 DEVICE_CLASS_CONNECTIVITY = "connectivity"
 DEVICE_CLASS_DOOR = "door"
 DEVICE_CLASS_GARAGE_DOOR = "garage_door"
-DEVICE_CLASS_GAS = "gas"
 DEVICE_CLASS_HEAT = "heat"
 DEVICE_CLASS_LIGHT = "light"
 DEVICE_CLASS_LOCK = "lock"
@@ -813,25 +868,37 @@ DEVICE_CLASS_PROBLEM = "problem"
 DEVICE_CLASS_SAFETY = "safety"
 DEVICE_CLASS_SMOKE = "smoke"
 DEVICE_CLASS_SOUND = "sound"
+DEVICE_CLASS_UPDATE = "update"
 DEVICE_CLASS_VIBRATION = "vibration"
 DEVICE_CLASS_WINDOW = "window"
 # device classes of both binary_sensor and sensor component
 DEVICE_CLASS_EMPTY = ""
 DEVICE_CLASS_BATTERY = "battery"
+DEVICE_CLASS_GAS = "gas"
 DEVICE_CLASS_POWER = "power"
 # device classes of sensor component
-DEVICE_CLASS_CARBON_MONOXIDE = "carbon_monoxide"
+DEVICE_CLASS_AQI = "aqi"
 DEVICE_CLASS_CARBON_DIOXIDE = "carbon_dioxide"
+DEVICE_CLASS_CARBON_MONOXIDE = "carbon_monoxide"
 DEVICE_CLASS_CURRENT = "current"
 DEVICE_CLASS_ENERGY = "energy"
 DEVICE_CLASS_HUMIDITY = "humidity"
 DEVICE_CLASS_ILLUMINANCE = "illuminance"
 DEVICE_CLASS_MONETARY = "monetary"
-DEVICE_CLASS_SIGNAL_STRENGTH = "signal_strength"
-DEVICE_CLASS_TEMPERATURE = "temperature"
+DEVICE_CLASS_NITROGEN_DIOXIDE = "nitrogen_dioxide"
+DEVICE_CLASS_NITROGEN_MONOXIDE = "nitrogen_monoxide"
+DEVICE_CLASS_NITROUS_OXIDE = "nitrous_oxide"
+DEVICE_CLASS_OZONE = "ozone"
+DEVICE_CLASS_PM1 = "pm1"
+DEVICE_CLASS_PM10 = "pm10"
+DEVICE_CLASS_PM25 = "pm25"
 DEVICE_CLASS_POWER_FACTOR = "power_factor"
 DEVICE_CLASS_PRESSURE = "pressure"
+DEVICE_CLASS_SIGNAL_STRENGTH = "signal_strength"
+DEVICE_CLASS_SULPHUR_DIOXIDE = "sulphur_dioxide"
+DEVICE_CLASS_TEMPERATURE = "temperature"
 DEVICE_CLASS_TIMESTAMP = "timestamp"
+DEVICE_CLASS_VOLATILE_ORGANIC_COMPOUNDS = "volatile_organic_compounds"
 DEVICE_CLASS_VOLTAGE = "voltage"
 
 # state classes
@@ -843,9 +910,7 @@ STATE_CLASS_MEASUREMENT = "measurement"
 # The state represents a total that only increases, a decrease is considered a reset.
 STATE_CLASS_TOTAL_INCREASING = "total_increasing"
 
-# This sensor does not support resetting. ie, it is not accumulative
-LAST_RESET_TYPE_NONE = ""
-# This sensor is expected to never reset its value
-LAST_RESET_TYPE_NEVER = "never"
-# This sensor may reset and Home Assistant will watch for this
-LAST_RESET_TYPE_AUTO = "auto"
+KEY_CORE = "core"
+KEY_TARGET_PLATFORM = "target_platform"
+KEY_TARGET_FRAMEWORK = "target_framework"
+KEY_FRAMEWORK_VERSION = "framework_version"
