@@ -1,29 +1,53 @@
 #include "esphome/core/helpers.h"
+#include "esphome/core/defines.h"
 #include <cstdio>
 #include <algorithm>
+#include <cmath>
+#include <cstring>
 
-#ifdef ARDUINO_ARCH_ESP8266
+#if defined(USE_ESP8266)
 #include <ESP8266WiFi.h>
-#else
+#include <osapi.h>
+#elif defined(USE_ESP32_FRAMEWORK_ARDUINO)
 #include <Esp.h>
+#elif defined(USE_ESP_IDF)
+#include "esp_system.h"
+#include <freertos/FreeRTOS.h>
+#include <freertos/portmacro.h>
+#endif
+#ifdef USE_ESP32_IGNORE_EFUSE_MAC_CRC
+#include "esp_efuse.h"
+#include "esp_efuse_table.h"
 #endif
 
 #include "esphome/core/log.h"
-#include "esphome/core/esphal.h"
+#include "esphome/core/hal.h"
 
 namespace esphome {
 
 static const char *const TAG = "helpers";
 
+void get_mac_address_raw(uint8_t *mac) {
+#ifdef USE_ESP32
+#ifdef USE_ESP32_IGNORE_EFUSE_MAC_CRC
+  // On some devices, the MAC address that is burnt into EFuse does not
+  // match the CRC that goes along with it. For those devices, this
+  // work-around reads and uses the MAC address as-is from EFuse,
+  // without doing the CRC check.
+  esp_efuse_read_field_blob(ESP_EFUSE_MAC_FACTORY, mac, 48);
+#else
+  esp_efuse_mac_get_default(mac);
+#endif
+#endif
+#ifdef USE_ESP8266
+  WiFi.macAddress(mac);
+#endif
+}
+
 std::string get_mac_address() {
   char tmp[20];
   uint8_t mac[6];
-#ifdef ARDUINO_ARCH_ESP32
-  esp_efuse_mac_get_default(mac);
-#endif
-#ifdef ARDUINO_ARCH_ESP8266
-  WiFi.macAddress(mac);
-#endif
+  get_mac_address_raw(mac);
   sprintf(tmp, "%02x%02x%02x%02x%02x%02x", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
   return std::string(tmp);
 }
@@ -31,22 +55,21 @@ std::string get_mac_address() {
 std::string get_mac_address_pretty() {
   char tmp[20];
   uint8_t mac[6];
-#ifdef ARDUINO_ARCH_ESP32
-  esp_efuse_mac_get_default(mac);
-#endif
-#ifdef ARDUINO_ARCH_ESP8266
-  WiFi.macAddress(mac);
-#endif
+  get_mac_address_raw(mac);
   sprintf(tmp, "%02X:%02X:%02X:%02X:%02X:%02X", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
   return std::string(tmp);
 }
 
+#ifdef USE_ESP32
+void set_mac_address(uint8_t *mac) { esp_base_mac_addr_set(mac); }
+#endif
+
 std::string generate_hostname(const std::string &base) { return base + std::string("-") + get_mac_address(); }
 
 uint32_t random_uint32() {
-#ifdef ARDUINO_ARCH_ESP32
+#ifdef USE_ESP32
   return esp_random();
-#else
+#elif defined(USE_ESP8266)
   return os_random();
 #endif
 }
@@ -55,6 +78,17 @@ double random_double() { return random_uint32() / double(UINT32_MAX); }
 
 float random_float() { return float(random_double()); }
 
+void fill_random(uint8_t *data, size_t len) {
+#if defined(USE_ESP_IDF) || defined(USE_ESP32_FRAMEWORK_ARDUINO)
+  esp_fill_random(data, len);
+#elif defined(USE_ESP8266)
+  int err = os_get_random(data, len);
+  assert(err == 0);
+#else
+#error "No random source for this system config"
+#endif
+}
+
 static uint32_t fast_random_seed = 0;  // NOLINT(cppcoreguidelines-avoid-non-const-global-variables)
 
 void fast_random_set_seed(uint32_t seed) { fast_random_seed = seed; }
@@ -114,10 +148,13 @@ std::string truncate_string(const std::string &s, size_t length) {
 }
 
 std::string value_accuracy_to_string(float value, int8_t accuracy_decimals) {
-  auto multiplier = float(powf(10.0f, accuracy_decimals));
-  float value_rounded = roundf(value * multiplier) / multiplier;
+  if (accuracy_decimals < 0) {
+    auto multiplier = powf(10.0f, accuracy_decimals);
+    value = roundf(value * multiplier) / multiplier;
+    accuracy_decimals = 0;
+  }
   char tmp[32];  // should be enough, but we should maybe improve this at some point.
-  dtostrf(value_rounded, 0, uint8_t(std::max(0, int(accuracy_decimals))), tmp);
+  snprintf(tmp, sizeof(tmp), "%.*f", accuracy_decimals, value);
   return std::string(tmp);
 }
 std::string uint64_to_string(uint64_t num) {
@@ -132,21 +169,6 @@ std::string uint32_to_string(uint32_t num) {
   snprintf(buffer, sizeof(buffer), "%04X%04X", address16[1], address16[0]);
   return std::string(buffer);
 }
-static char *global_json_build_buffer = nullptr;  // NOLINT(cppcoreguidelines-avoid-non-const-global-variables)
-static size_t global_json_build_buffer_size = 0;  // NOLINT(cppcoreguidelines-avoid-non-const-global-variables)
-
-void reserve_global_json_build_buffer(size_t required_size) {
-  if (global_json_build_buffer_size == 0 || global_json_build_buffer_size < required_size) {
-    delete[] global_json_build_buffer;
-    global_json_build_buffer_size = std::max(required_size, global_json_build_buffer_size * 2);
-
-    size_t remainder = global_json_build_buffer_size % 16U;
-    if (remainder != 0)
-      global_json_build_buffer_size += 16 - remainder;
-
-    global_json_build_buffer = new char[global_json_build_buffer_size];
-  }
-}
 
 ParseOnOffState parse_on_off(const char *str, const char *on, const char *off) {
   if (on == nullptr && strcasecmp(str, "on") == 0)
@@ -264,6 +286,39 @@ optional<int> parse_int(const std::string &str) {
     return {};
   return value;
 }
+
+optional<int> parse_hex(const char chr) {
+  int out = chr;
+  if (out >= '0' && out <= '9')
+    return (out - '0');
+  if (out >= 'A' && out <= 'F')
+    return (10 + (out - 'A'));
+  if (out >= 'a' && out <= 'f')
+    return (10 + (out - 'a'));
+  return {};
+}
+
+optional<int> parse_hex(const std::string &str, size_t start, size_t length) {
+  if (str.length() < start) {
+    return {};
+  }
+  size_t end = start + length;
+  if (str.length() < end) {
+    return {};
+  }
+  int out = 0;
+  for (size_t i = start; i < end; i++) {
+    char chr = str[i];
+    auto digit = parse_hex(chr);
+    if (!digit.has_value()) {
+      ESP_LOGW(TAG, "Can't convert '%s' to number, invalid character %c!", str.substr(start, length).c_str(), chr);
+      return {};
+    }
+    out = (out << 4) | *digit;
+  }
+  return out;
+}
+
 uint32_t fnv1_hash(const std::string &str) {
   uint32_t hash = 2166136261UL;
   for (char c : str) {
@@ -312,6 +367,21 @@ bool str_startswith(const std::string &full, const std::string &start) { return
 bool str_endswith(const std::string &full, const std::string &ending) {
   return full.rfind(ending) == (full.size() - ending.size());
 }
+std::string str_sprintf(const char *fmt, ...) {
+  std::string str;
+  va_list args;
+
+  va_start(args, fmt);
+  size_t length = vsnprintf(nullptr, 0, fmt, args);
+  va_end(args);
+
+  str.resize(length);
+  va_start(args, fmt);
+  vsnprintf(&str[0], length + 1, fmt, args);
+  va_end(args);
+
+  return str;
+}
 
 uint16_t encode_uint16(uint8_t msb, uint8_t lsb) { return (uint16_t(msb) << 8) | uint16_t(lsb); }
 std::array<uint8_t, 2> decode_uint16(uint16_t value) {
@@ -340,13 +410,76 @@ std::string hexencode(const uint8_t *data, uint32_t len) {
   return res;
 }
 
-#ifdef ARDUINO_ARCH_ESP8266
-ICACHE_RAM_ATTR InterruptLock::InterruptLock() { xt_state_ = xt_rsil(15); }
-ICACHE_RAM_ATTR InterruptLock::~InterruptLock() { xt_wsr_ps(xt_state_); }
+void rgb_to_hsv(float red, float green, float blue, int &hue, float &saturation, float &value) {
+  float max_color_value = std::max(std::max(red, green), blue);
+  float min_color_value = std::min(std::min(red, green), blue);
+  float delta = max_color_value - min_color_value;
+
+  if (delta == 0)
+    hue = 0;
+  else if (max_color_value == red)
+    hue = int(fmod(((60 * ((green - blue) / delta)) + 360), 360));
+  else if (max_color_value == green)
+    hue = int(fmod(((60 * ((blue - red) / delta)) + 120), 360));
+  else if (max_color_value == blue)
+    hue = int(fmod(((60 * ((red - green) / delta)) + 240), 360));
+
+  if (max_color_value == 0)
+    saturation = 0;
+  else
+    saturation = delta / max_color_value;
+
+  value = max_color_value;
+}
+
+void hsv_to_rgb(int hue, float saturation, float value, float &red, float &green, float &blue) {
+  float chroma = value * saturation;
+  float hue_prime = fmod(hue / 60.0, 6);
+  float intermediate = chroma * (1 - fabs(fmod(hue_prime, 2) - 1));
+  float delta = value - chroma;
+
+  if (0 <= hue_prime && hue_prime < 1) {
+    red = chroma;
+    green = intermediate;
+    blue = 0;
+  } else if (1 <= hue_prime && hue_prime < 2) {
+    red = intermediate;
+    green = chroma;
+    blue = 0;
+  } else if (2 <= hue_prime && hue_prime < 3) {
+    red = 0;
+    green = chroma;
+    blue = intermediate;
+  } else if (3 <= hue_prime && hue_prime < 4) {
+    red = 0;
+    green = intermediate;
+    blue = chroma;
+  } else if (4 <= hue_prime && hue_prime < 5) {
+    red = intermediate;
+    green = 0;
+    blue = chroma;
+  } else if (5 <= hue_prime && hue_prime < 6) {
+    red = chroma;
+    green = 0;
+    blue = intermediate;
+  } else {
+    red = 0;
+    green = 0;
+    blue = 0;
+  }
+
+  red += delta;
+  green += delta;
+  blue += delta;
+}
+
+#ifdef USE_ESP8266
+IRAM_ATTR InterruptLock::InterruptLock() { xt_state_ = xt_rsil(15); }
+IRAM_ATTR InterruptLock::~InterruptLock() { xt_wsr_ps(xt_state_); }
 #endif
-#ifdef ARDUINO_ARCH_ESP32
-ICACHE_RAM_ATTR InterruptLock::InterruptLock() { portDISABLE_INTERRUPTS(); }
-ICACHE_RAM_ATTR InterruptLock::~InterruptLock() { portENABLE_INTERRUPTS(); }
+#ifdef USE_ESP32
+IRAM_ATTR InterruptLock::InterruptLock() { portDISABLE_INTERRUPTS(); }
+IRAM_ATTR InterruptLock::~InterruptLock() { portENABLE_INTERRUPTS(); }
 #endif
 
 }  // namespace esphome
