@@ -6,16 +6,12 @@
 #include <memory>
 #include <type_traits>
 
-#include "esphome/core/optional.h"
-#include "esphome/core/esphal.h"
-
-#ifdef CLANG_TIDY
-#undef ICACHE_RAM_ATTR
-#define ICACHE_RAM_ATTR
-#undef ICACHE_RODATA_ATTR
-#define ICACHE_RODATA_ATTR
+#ifdef USE_ESP32_FRAMEWORK_ARDUINO
+#include "esp32-hal-psram.h"
 #endif
 
+#include "esphome/core/optional.h"
+
 #define HOT __attribute__((hot))
 #define ESPDEPRECATED(msg, when) __attribute__((deprecated(msg)))
 #define ALWAYS_INLINE __attribute__((always_inline))
@@ -30,11 +26,21 @@ namespace esphome {
 /// The characters that are allowed in a hostname.
 extern const char *const HOSTNAME_CHARACTER_ALLOWLIST;
 
-/// Gets the MAC address as a string, this can be used as way to identify this ESP.
+/// Read the raw MAC address into the provided byte array (6 bytes).
+void get_mac_address_raw(uint8_t *mac);
+
+/// Get the MAC address as a string, using lower case hex notation.
+/// This can be used as way to identify this ESP.
 std::string get_mac_address();
 
+/// Get the MAC address as a string, using colon-separated upper case hex notation.
 std::string get_mac_address_pretty();
 
+#ifdef USE_ESP32
+/// Set the MAC address to use from the provided byte array (6 bytes).
+void set_mac_address(uint8_t *mac);
+#endif
+
 std::string to_string(const std::string &val);
 std::string to_string(int val);
 std::string to_string(long val);                // NOLINT
@@ -47,7 +53,8 @@ std::string to_string(double val);
 std::string to_string(long double val);
 optional<float> parse_float(const std::string &str);
 optional<int> parse_int(const std::string &str);
-
+optional<int> parse_hex(const std::string &str, size_t start, size_t length);
+optional<int> parse_hex(char chr);
 /// Sanitize the hostname by removing characters that are not in the allowlist and truncating it to 63 chars.
 std::string sanitize_hostname(const std::string &hostname);
 
@@ -62,6 +69,9 @@ bool str_equals_case_insensitive(const std::string &a, const std::string &b);
 bool str_startswith(const std::string &full, const std::string &start);
 bool str_endswith(const std::string &full, const std::string &ending);
 
+/// sprintf-like function returning std::string instead of writing to char array.
+std::string __attribute__((format(printf, 1, 2))) str_sprintf(const char *fmt, ...);
+
 class HighFrequencyLoopRequester {
  public:
   void start();
@@ -92,10 +102,15 @@ template<typename T> T clamp(T val, T min, T max);
  */
 float lerp(float completion, float start, float end);
 
-/// std::make_unique
+// Not all platforms we support target C++14 yet, so we can't unconditionally use std::make_unique. Provide our own
+// implementation if needed, and otherwise pull std::make_unique into scope so that we have a uniform API.
+#if __cplusplus >= 201402L
+using std::make_unique;
+#else
 template<typename T, typename... Args> std::unique_ptr<T> make_unique(Args &&...args) {
   return std::unique_ptr<T>(new T(std::forward<Args>(args)...));
 }
+#endif
 
 /// Return a random 32 bit unsigned integer.
 uint32_t random_uint32();
@@ -109,6 +124,8 @@ double random_double();
 /// Returns a random float between 0 and 1. Essentially just casts random_double() to a float.
 float random_float();
 
+void fill_random(uint8_t *data, size_t len);
+
 void fast_random_set_seed(uint32_t seed);
 uint32_t fast_random_32();
 uint16_t fast_random_16();
@@ -142,13 +159,18 @@ std::array<uint8_t, 2> decode_uint16(uint16_t value);
 /// Encode a 32-bit unsigned integer given four bytes in MSB -> LSB order
 uint32_t encode_uint32(uint8_t msb, uint8_t byte2, uint8_t byte3, uint8_t lsb);
 
+/// Convert RGB floats (0-1) to hue (0-360) & saturation/value percentage (0-1)
+void rgb_to_hsv(float red, float green, float blue, int &hue, float &saturation, float &value);
+/// Convert hue (0-360) & saturation/value percentage (0-1) to RGB floats (0-1)
+void hsv_to_rgb(int hue, float saturation, float value, float &red, float &green, float &blue);
+
 /***
  * An interrupt helper class.
  *
  * This behaves like std::lock_guard. As long as the value is visible in the current stack, all interrupts
  * (including flash reads) will be disabled.
  *
- * Please note all functions called when the interrupt lock must be marked ICACHE_RAM_ATTR (loading code into
+ * Please note all functions called when the interrupt lock must be marked IRAM_ATTR (loading code into
  * instruction cache is done via interrupts; disabling interrupts prevents data not already in cache from being
  * pulled from flash).
  *
@@ -170,7 +192,7 @@ class InterruptLock {
   ~InterruptLock();
 
  protected:
-#ifdef ARDUINO_ARCH_ESP8266
+#ifdef USE_ESP8266
   uint32_t xt_state_;
 #endif
 };
@@ -274,8 +296,8 @@ template<typename T, typename... X> class TemplatableValue {
     LAMBDA,
   } type_;
 
-  T value_;
-  std::function<T(X...)> f_;
+  T value_{};
+  std::function<T(X...)> f_{};
 };
 
 template<typename... X> class TemplatableStringValue : public TemplatableValue<std::string, X...> {
@@ -326,14 +348,14 @@ uint32_t fnv1_hash(const std::string &str);
 
 template<typename T> T *new_buffer(size_t length) {
   T *buffer;
-#ifdef ARDUINO_ARCH_ESP32
+#ifdef USE_ESP32_FRAMEWORK_ARDUINO
   if (psramFound()) {
     buffer = (T *) ps_malloc(length);
   } else {
-    buffer = new T[length];
+    buffer = new T[length];  // NOLINT(cppcoreguidelines-owning-memory)
   }
 #else
-  buffer = new T[length];
+  buffer = new T[length];  // NOLINT(cppcoreguidelines-owning-memory)
 #endif
 
   return buffer;
