@@ -52,9 +52,7 @@ class ProgressBar:
             return
         self.last_progress = new_progress
         block = int(round(bar_length * progress))
-        text = "\rUploading: [{0}] {1}% {2}".format(
-            "=" * block + " " * (bar_length - block), new_progress, status
-        )
+        text = f"\rUploading: [{'=' * block + ' ' * (bar_length - block)}] {new_progress}% {status}"
         sys.stderr.write(text)
         sys.stderr.flush()
 
@@ -154,7 +152,7 @@ def check_error(data, expect):
     if not isinstance(expect, (list, tuple)):
         expect = [expect]
     if dat not in expect:
-        raise OTAError("Unexpected response from ESP: 0x{:02X}".format(data[0]))
+        raise OTAError(f"Unexpected response from ESP: 0x{data[0]:02X}")
 
 
 def send_check(sock, data, msg):
